<?php

namespace Drupal\s3fs_streamwrapper\Compiler;

use Drupal\s3fs_streamwrapper\StreamWrapper\S3fsBucketStream;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;

/**
 * Create container entries for the schemes that are enabled.
 *
 * The core RegisterStreamWrapperPass() must be called after this pass to
 * actually register the container entries created with this pass.
 */
class S3fsRegisterStreamWrappers implements CompilerPassInterface {

  /**
   * {@inheritdoc}
   */
  public function process(ContainerBuilder $container): void {
    /** @var \Drupal\Core\Config\CachedStorage $config_storage */
    $config_storage = $container->get('config.storage');
    $entity_config_names = $config_storage->listAll('s3fs_streamwrapper.s3fs_streamwrapper');
    $configured_entities = $config_storage->readMultiple($entity_config_names);

    foreach ($configured_entities as $key) {
      if (empty($key)) {
        // This should never happen, however if it does skip this entry.
        continue;
      }

      if ($key['status'] !== TRUE) {
        // Scheme is not enabled.
        continue;
      }

      $service_definition = new Definition(S3fsBucketStream::class);
      $service_definition->setTags(
        [
          'stream_wrapper' => [
            [
              'scheme' => $key['id'],
            ],
          ],
        ],
      );
      $service_definition->setPublic(TRUE);
      $container->setDefinition('stream_wrapper.' . $key['id'], $service_definition);

    }
  }

}
